﻿using UnityEngine;
using System.Collections;
using System;

public class LogicBase : MonoBehaviour {

	public static Action<LogicBase> OnInitialize;
	public static Action<LogicBase> OnStartGame;
	public static Action<LogicBase> OnGameWin;
	public static Action<LogicBase> OnGameOver;
	public static Action<LogicBase> OnRightMove;
	public static Action<HeroController> OnHeroDie;
	public static Action<string> OnDialog;

	private static LogicBase _instance;
	public static LogicBase instance { get { return _instance; } }
	public int iteration { get { return iterationID; } }
	public int currLevelID;
	public int nextLevelID;
	public string[] keyDialogs;
	public HeroController hero1;
	public HeroController hero2;
	public Transform spawnHeroPoint1;
	public Transform spawnHeroPoint2;
	public GameObject spawnEffect1;
	public GameObject spawnEffect2;
	public GameObject despawnEffect1;
	public GameObject despawnEffect2;
	public float despawnRange = 2f;
	public Transform despawnTransform1;
	public Transform despawnTransform2;
	public GameObject[] rightIndicator;

	protected int iterationID = -1;

	protected virtual void GameWin()
	{
		SceneFader.Load(nextLevelID, false);
	}

	protected virtual void GameOver()
	{
		Time.timeScale = 0f;
		SceneFader.Load(currLevelID, false);
	}

	protected virtual void Initialize()
	{
		for (int i = 0; i < rightIndicator.Length; i++)
			rightIndicator[i].SetActive(false);
		Time.timeScale = 1f;
	}

	protected virtual void DeInitialize()
	{ }

	protected virtual void WhenUpdate()
	{ }

	public virtual void StartPlay()
	{
		hero1.myTransform.position = spawnHeroPoint1.position;
		hero2.myTransform.position = spawnHeroPoint2.position;
		if (OnDialog != null && keyDialogs != null && keyDialogs.Length > 0)
		{
			for (int i = 0; i < keyDialogs.Length; i++)
			{
				if(!string.IsNullOrEmpty(keyDialogs[i]))
					OnDialog(keyDialogs[i]);
			}
			
		}
	}

	public virtual void KillHero(HeroController hero)
	{
		if (hero == hero1)
		{
			hero1.lives--;
			hero1.myTransform.position = spawnHeroPoint1.position;
			StartCoroutine(CoOnTimer(spawnEffect1));
			if (hero1.lives > 0)
				hero1.Spawn();
		}
		else if (hero == hero2)
		{
			hero2.lives--;
			hero2.myTransform.position = spawnHeroPoint2.position;
			StartCoroutine(CoOnTimer(spawnEffect2));
			if (hero2.lives > 0)
				hero2.Spawn();
		}
		if (OnHeroDie != null)
			OnHeroDie(hero);

		if (hero1.lives < 1 || hero2.lives < 1)
		{
			GameOver();
		}
	}



	public virtual void RightMove()
	{
		iterationID++;
		if (iterationID >= 0)
		{
			if (iterationID < rightIndicator.Length)
			{
				rightIndicator[iterationID].SetActive(true);
			}
			else
			{
				StartCoroutine(ActivateOut());
			}
		}
		if (OnRightMove != null)
			OnRightMove(this);
	}

	IEnumerator ActivateOut()
	{
		despawnEffect1.SetActive(true);
		despawnEffect2.SetActive(true);
		hero1.SetFollow(despawnTransform1);
		hero2.SetFollow(despawnTransform2);
		while (true)
		{
			if (
				((Vector3.Distance(hero1.myTransform.position, despawnTransform1.position) < despawnRange) &&
					(Vector3.Distance(hero2.myTransform.position, despawnTransform2.position) < despawnRange)) ||
				((Vector3.Distance(hero1.myTransform.position, despawnTransform2.position) < despawnRange) &&
					(Vector3.Distance(hero2.myTransform.position, despawnTransform1.position) < despawnRange))
			)
			{
				GameWin();
				yield break;
			}
			else
				yield return null;
		}
	}

	protected IEnumerator CoOnTimer(GameObject go)
	{
		go.SetActive(true);
		yield return new WaitForSeconds(3f);
		go.SetActive(false);
	}

	#region MonoBehaviour
	void OnEnable()
	{
		if (_instance == null)
		{
			_instance = this;
		}
	}

	void OnDisable()
	{
		if (_instance == this)
		{
			_instance = null;
		}
	}

	void Start()
	{
		Initialize();
		StartPlay();

	}

	void Update()
	{
		WhenUpdate();
	}

	#endregion
}
