﻿using UnityEngine;
using System;
using System.Collections;
using System.Collections.Generic;


// By Sergey Vitukhin. 20150205.
public class PiLang : MonoBehaviour
{
    #region Clases
    [System.Serializable]
    public class OneLangResource
    {
        public SystemLanguage name;
        public string path;
    }
    #endregion

    #region Actions
    public static System.Action OnLangChanged;
    #endregion

    #region Static Members
    private static PiLang _instance = null;
    private static readonly string _name = "PiLang";
    private static bool _isInitialized = false;
    #endregion

    public SystemLanguage startLang = SystemLanguage.English;
    public OneLangResource[] languages;

    private Dictionary<string, string> _dictionary = null;
    private SystemLanguage _currentLang = SystemLanguage.Unknown;

    #region Static Methods
    public static void Initialize()
    {
        if (_isInitialized)
            return;

        Instantiate(Resources.Load(_name));
        if (_isInitialized && _instance != null)
        {
            bool res = false;
            int obj;

			obj = PlayerPrefs.GetInt(_name, -1);

            if (obj >= 0)
            {
                res = SetLanguage((SystemLanguage)Convert.ToInt32(obj));
            }
            else
            {
                res = SetLanguage(Application.systemLanguage);
            }
            if (!res)
            {
                SetLanguage(_instance.startLang);
            }
        }
    }

    public static bool SetLanguage(SystemLanguage language)
    {
        if (!_isInitialized)
            Initialize();

        if (_isInitialized && _instance != null)
        {
            return _instance.ChangeLanguage(language);
        }
        else
        {
            return false;
        }
    }

    public static SystemLanguage GetCurrentLanguage()
    {
        if (!_isInitialized)
            Initialize();

        if (_isInitialized && _instance != null)
        {
            return _instance._currentLang;
        }
        else
        {
            return SystemLanguage.English;
        }
    }

    public static string Localize(string key)
    {
        if (!_isInitialized)
            Initialize();
        
        string result = null;

        if (_isInitialized && _instance != null)
        {
            if (_instance._dictionary.TryGetValue(key, out result))
            {}
            else
            {
#if UNITY_EDITOR
                Debug.LogWarning(string.Format("Localization key not found: {0}", key));
#endif
                result = key;
            }
        }
        return result;
    }
    #endregion

    #region MonoBehaviour
    private void Awake()
    {
        if (!_isInitialized && _instance == null)
        {
            _instance = this;
            name = _name;
            DontDestroyOnLoad(gameObject);
            _isInitialized = true;
        }
        else
        {
            Destroy(gameObject);
        }
    }

    private void OnDestroy()
    {
        if (_instance == this)
        {
            _instance = null;
            _dictionary = null;
            languages = null;
            OnLangChanged = null;
        }
    }
    #endregion

    private bool ChangeLanguage(SystemLanguage lang)
    {
        bool result = false;
        if (_instance != null && languages != null && languages.Length > 0 && _currentLang != lang)
        {
            for (int i = 0; i < languages.Length; ++i)
            {
                if (languages[i].name == lang && LoadFileLang(languages[i].path))
                {
                    _currentLang = lang;
					PlayerPrefs.SetInt(_name, (int)_currentLang);
                    _isInitialized = true;
                    result = true;
                    break;
                }
            }
        }
        return result;
    }

    private bool LoadFileLang(string path)
    {
        if (string.IsNullOrEmpty(path))
            return false;
        TextAsset asset = Resources.Load(path, typeof(TextAsset)) as TextAsset;
        
        if (asset)
        {
            string[] lines;
            string[] kv;
            string key;
            string value;
            if (_dictionary == null)
                _dictionary = new Dictionary<string, string>();
            else
                _dictionary.Clear();

            lines = asset.text.Split('\n');

            for (int i = 0; i < lines.Length; ++i)
            {
                kv = lines[i].Split('=');
                if (kv.Length == 2)
                {
                    key = kv[0].Trim();
                    value = kv[1].Trim().Replace("\\n", "\n");
                    _dictionary[key] = value;
                }
            }
            Resources.UnloadAsset(asset);
            return true;
        }
        else
            return false;
    }

}
